#!/usr/bin/env python2.6
# David D Lowe (c) 2008-2009
# licensed under GPLv3
"""Epidermis creator is a GUI program for creating pigments"""

import pygtk
pygtk.require("2.0")
import gtk, gtk.glade
import gobject
if __name__ == "__main__":
    gobject.threads_init()
import os, sys
if 'pychecker' not in sys.argv[0]:
    import gtkexcepthook
import const
import handy
if not handy.determine_path() in sys.path:
    sys.path.insert(1, handy.determine_path())
import subprocess
import threading
import locale
import gconf
import managepigments
import pigments.pigment as pigment
from gettext import gettext as _
import gettext

# TODO: warn user if she tries to select gtk-2.0 or metacity-1 as theme directory
# TODO: menu copy, paste, cut...

class CreatorApp:
    
    # consts:
    
    iSELECT_PICTURE = 0
    iSELECT_NAME = 1
    iSELECT_PIGMENT = 2
    
    
    def destroy_event(self, widget, data=None):
        """When the main widow is closed, this function quits the application"""
        if const.DEBUG:
            "Window closed, exiting"
        gtk.main_quit()
        sys.exit()
        return False
    
    def __init__(self):
        """Initialize CreatorApp"""
        # Load the glade file
        if os.path.exists(os.path.join(determine_path(), "data", "creator.glade")):
            print "using custom creator.glade"
            self.wTreeFileName = os.path.join(determine_path() ,"data", "creator.glade")
        else:
            self.wTreeFileName = "/usr/share/epidermis/data/creator.glade"
            
        self.wTree = gtk.glade.XML(self.wTreeFileName)
        
      
        # reference gtk objects
        self.win = self.wTree.get_widget("winEpidermisCreator")
        self.winSelectPigment = self.wTree.get_widget("dialogSelectPigment")
        self.winSelectTheme = self.wTree.get_widget("dialogSelectTheme")
        
        widgetNames =  ("frameSpecific", "tableSkin", "comboboxPigmentType", \
        "comboboxentryCopyright", "comboboxPictureOptions", "comboboxColourShadingType", \
        "menuNew", "menuOpen", "menuSave", "menuQuit", "menuAbout", "entryName", "entryCodename", \
        "entryAuthor", "scrolledWindowDescription", "textviewDescription", "filechooserbuttonPreview", \
        "filechooserbuttonFile", "treeviewSelectPigment", "butSelectOpen", "btSelectCancel", \
        "labelFile", "toolbuttonNew", "toolbuttonSave", "toolbuttonOpen",  "imagePreview", \
        "buttonAutogenerate","butSelectThemeOpen","butSelectThemeCancel","treeviewSelectTheme")
        
        # the following few lines are added because pychecker doesn't understand setattr
        noneList = []
        for name in widgetNames:
            noneList.append(None)
        noneTuple = tuple(noneList)
        (self.frameSpecific, self.tableSkin, self.comboboxPigmentType, self.comboboxentryCopyright, self.comboboxPictureOptions, self.comboboxColourShadingType, self.menuNew, self.menuOpen, self.menuSave, self.menuQuit, self.menuAbout, self.entryName, self.entryCodename, self.entryAuthor, self.scrolledWindowDescription, self.textviewDescription, self.filechooserbuttonPreview, self.filechooserbuttonFile, self.treeviewSelectPigment, self.butSelectOpen, self.btSelectCancel, self.labelFile, self.toolbuttonNew, self.toolbuttonSave, self.toolbuttonOpen, self.imagePreview, self.buttonAutogenerate,self.butSelectThemeOpen,self.butSelectThemeCancel,self.treeviewSelectTheme) = noneTuple
        
        for bit in widgetNames:
            setattr(self, bit, self.wTree.get_widget(bit))
        
        # listStorePigmentTypes: code name, display name
        self.listStorePigmentTypes = gtk.ListStore(str, str)
        for pt in const.PIGMENT_TYPES:
            self.listStorePigmentTypes.append([pt,self.get_display_name_of_pigment_type(pt)])
        self.comboboxPigmentType.set_model(self.listStorePigmentTypes)
        cell = gtk.CellRendererText()
        self.comboboxPigmentType.pack_start(cell, True)
        self.comboboxPigmentType.add_attribute(cell, "text", 1)
        
        # listStoreCopyright: just display name
        self.listStoreCopyright = gtk.ListStore(str)
        for bit in ("GPL", "Creative Commons", "BSD", "Public Domain"):
            self.listStoreCopyright.append([bit])
        self.comboboxentryCopyright.set_model(self.listStoreCopyright)
        self.comboboxentryCopyright.set_text_column(0)
        
        # listStorePictureOptions:
        self.listStorePictureOptions = gtk.ListStore(str)
        for bit in ("none", "wallpaper", "centered", "scaled", "stretched", "zoom"):
            self.listStorePictureOptions.append([bit])
        self.comboboxPictureOptions.set_model(self.listStorePictureOptions)
        cell1 = gtk.CellRendererText()
        self.comboboxPictureOptions.pack_start(cell1, True)
        self.comboboxPictureOptions.add_attribute(cell1, "text", 0)
        self.comboboxPictureOptions.set_active(5)
        
        # listStoreColourShadingType:
        self.listStoreColourShadingType = gtk.ListStore(str)
        for bit in ("solid", "horizontal-gradient", "vertical-gradient"):
            self.listStoreColourShadingType.append([bit])
        self.comboboxColourShadingType.set_model(self.listStoreColourShadingType)
        cell2 = gtk.CellRendererText()
        self.comboboxColourShadingType.pack_start(cell2, True)
        self.comboboxColourShadingType.add_attribute(cell2, "text", 0)
        self.comboboxColourShadingType.set_active(0)
        
        # file filters:
        imageFilter = gtk.FileFilter()
        for mime_type in ("image/jpeg","image/png", "image/svg+xml", "image/svg"):
            imageFilter.add_mime_type(mime_type)
        imageFilter.set_name(_("Image files"))
        allFilter = gtk.FileFilter()
        allFilter.add_pattern("*")
        allFilter.set_name(_("All files"))
        self.filechooserbuttonPreview.add_filter(imageFilter)
        self.filechooserbuttonPreview.add_filter(allFilter)
        
        # connect signals
        self.win.connect("destroy", self.destroy_event)
        self.comboboxPigmentType.connect("changed", self.comboboxPigmentType_callback)
        self.menuNew.connect("activate", lambda unused: self.new())
        self.toolbuttonNew.connect("clicked", lambda unused: self.new())
        self.menuOpen.connect("activate", lambda unused: self.open_callback())
        self.menuSave.connect("activate", self.menuSave_callback)
        self.toolbuttonSave.connect("clicked", self.menuSave_callback)
        self.toolbuttonOpen.connect("clicked", lambda unused:self.open_callback())
        self.menuQuit.connect("activate", self.menuQuit_callback)
        self.menuAbout.connect("activate", self.menuAbout_callback)
        self.entryName.connect("changed", self.entryName_callback)
        self.entryCodename.connect("changed", self.entryCodename_callback)
        #self.filechooserbuttonFile.connect("selection-changed",self.filechooserbuttonFile_callback)
        self.filechooserbuttonPreview.connect("selection-changed", self.filechooserbuttonPreview_callback)
        self.buttonAutogenerate.connect("clicked", self.buttonAutogenerate_callback)
        for pt in const.PIGMENT_TYPES[1:]:
            self.wTree.get_widget("button" + pt.capitalize()).connect("clicked", self.pigment_selector_callback, pt)
        for ii in ("Gtk","Icons","Cursors"):
            self.wTree.get_widget("buttonPreferred" + ii).connect("clicked", self.preferred_callback, ii)
        
        # set program class
        gtk.gdk.set_program_class("EpidermisCreator")
        
        # get gconf settings
        client = gconf.client_get_default()
        self.repoDir = client.get_string("/apps/epidermis/creator/repo_dir")
        self.pigmentDir = os.path.join(const.DATA_HOME, "epidermis")
        if self.repoDir is None:
            self.repoDir = os.path.join(const.DATA_HOME, "epidermis", "repo")
            subprocess.call(["mkdir", "-p", self.repoDir])
            client.set_string("/apps/epidermis/creator/repo_dir", self.repoDir)
        
        # initialize SizeGroups
        self.sizeGroup0 = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        self.sizeGroup1 = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        
        # initialize variables
        self.codenameAutomatic = True
        self.codenameManual = False
        self.openDir = os.path.expanduser("~")
        self.pigmentPreviewPx = {}
        for pt in const.PIGMENT_TYPES:
            self.pigmentPreviewPx[pt] = None        
        
        # set skin as default pigment type
        self.currentFrameSpecificChild = None
        self.comboboxPigmentType.set_active(0)
        self.comboboxPigmentType_callback(self.comboboxPigmentType)

        #self.win.resize(600, 600)
        self.win.show_all()
    
    def new(self):
        
        # text entries:
        for nn in ("entryName", "entryCodename", "entryAuthor", "entryPrimaryColours", \
        "entrySecondaryColours", "entryKeyWords", "entryVersion", "entryWebpage"):
            self.wTree.get_widget(nn).set_text("")
        # comboboxentry:
        self.wTree.get_widget("comboboxentryCopyright").get_child().set_text("")
        # file choosers:
        for nn in ("filechooserbuttonPreview", "filechooserbuttonFile"):
            self.wTree.get_widget(nn).unselect_all()
        # pigment choosers:
        for nn in const.PIGMENT_TYPES[1:]:
            self.pigment_selector_set_text(self.wTree.get_widget("button" + nn.capitalize()), "")
        # comboboxs:
        for widget in (self.comboboxColourShadingType, self.comboboxPictureOptions, self.comboboxPigmentType):
            widget.set_active(0)
        # colour choosers:
        for nn in ("colorbuttonPrimaryColour", "colorbuttonSecondaryColour"):
            self.wTree.get_widget(nn).set_color(gtk.gdk.Color(0,0,0))
        # text view
        self.textviewDescription.get_buffer().set_text("")
    
    def open_callback(self):
        # Open GTK file dialog and get results
        filter = gtk.FileFilter()
        filter.add_pattern("*.pigment")
        filter.set_name(_("pigments"))
        
        dd = gtk.FileChooserDialog(_("Open pigments"), None, gtk.FILE_CHOOSER_ACTION_OPEN, \
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_OPEN, gtk.RESPONSE_ACCEPT))
        dd.add_filter(filter)
        dd.set_current_folder(self.openDir)
        res = dd.run()
        ff = dd.get_filenames()
        self.openDir = dd.get_current_folder()
        dd.destroy()
        if res == gtk.RESPONSE_ACCEPT:
            self.open([ff[0]])
        
    def open(self,filenames):
        for ff in filenames:
            self.new()
            filename = os.path.abspath(ff)
            pigml = pigment.Pigment()
            pigml.load(filename)
            
            pigm = managepigments.create_pigment(pigml.type.codeName)
            pigm.load(filename)
            
            # set pigment type
            ptNumber = -1
            for litem in xrange(len(self.listStorePigmentTypes)):
                if self.listStorePigmentTypes[litem][0] == pigm.type.codeName:
                    ptNumber = litem
                    break
            if ptNumber < 0:
                self.show_warning(_("Pigment type not supported: %s") % pigm.type.codeName)
                return
            self.comboboxPigmentType.set_active(ptNumber)
            
            # basic info
            self.entryName.set_text(pigm.get_human_name())
            self.entryCodename.set_text(pigm.codeName)
            buff = self.textviewDescription.get_buffer()
            encoding = locale.getpreferredencoding()
            utf8conv = lambda x : unicode(x, encoding).encode('utf8')
            buff.set_text(utf8conv(str(pigm.get_description())))
            self.comboboxentryCopyright.get_child().set_text(pigm.copyright)
            self.entryAuthor.set_text(pigm.author)
            
            # preview
            previewPixBuf = pigm.get_preview()
            subprocess.call(["mkdir", "-p", "/tmp/epidermiscreator"])
            if len(pigm.previewFilename) > 0:
                justFilename = pigm.previewFilename.split("/")[-1]
            else:
                justFilename = pigm.codeName + "_preview.png"
            previewFilename = os.path.join("/tmp/epidermiscreator", justFilename)
            saveFiletypes = {"jpg": "jpeg", "jpeg":"jpeg", "png":"png"}
            try:
                previewPixBuf.save(previewFilename, saveFiletypes.get(justFilename.split(".")[-1].lower(), "png"))
                self.filechooserbuttonPreview.set_filename(previewFilename)
            except:
                if pigm.type.codeName == "skin":
                    self.filechooserbuttonPreview.set_filename("")
                else:
                    raise
            
            # attachment
            if len(pigm.attachments) > 0:
                self.filechooserbuttonFile.set_filename(pigm.attachments[0])
                if const.DEBUG:
                    print "open pigment with attachment: ", pigm.attachments[0]
            
                
            getw = self.wTree.get_widget
            if pigm.type.codeName == "skin":
                getw("entryPrimaryColours").set_text(pigm.get_words(pigm.primaryColours))
                getw("entrySecondaryColours").set_text(pigm.get_words(pigm.secondaryColours))
                getw("entryKeyWords").set_text(pigm.get_words(pigm.keyWords))
                getw("entryVersion").set_text(pigm.version)
                getw("entryWebpage").set_text(pigm.webPage)
                for bit in const.PIGMENT_TYPES[1:]:
                    if pigm.linkedPigments.has_key(bit):
                        self.pigment_selector_set_text(getw("button" + bit.capitalize()), pigm.linkedPigments[bit].codeName)
                    else:
                        self.pigment_selector_set_text(getw("button" + bit.capitalize()), "")
            elif pigm.type.codeName == "wallpaper":
                poItemN = -1
                for litem in xrange(len(self.listStorePictureOptions)):
                    if self.listStorePictureOptions[litem][0] == pigm.pictureOptions:
                        poItemN = litem
                        break
                if poItemN < 0:
                    print >> sys.stderr, pigm.pictureOptions + " is not a recognised picture option"
                else:
                    self.comboboxPictureOptions.set_active(poItemN)
                csItemN = -1
                for litem in xrange(len(self.listStoreColourShadingType)):
                    if self.listStoreColourShadingType[litem][0] == pigm.colourShadingType:
                        csItemN = litem
                        break
                if csItemN < 0:
                    print >> sys.stderr, pigm.colourShadingType + " is not a recoginsed color shading type"
                else:
                    self.comboboxColourShadingType.set_active(csItemN)
                try:
                    getw("colorbuttonPrimaryColour").set_color(gtk.gdk.color_parse(pigm.primaryColour))
                    if len(pigm.secondaryColour) > 0:
                        getw("colorbuttonSecondaryColour").set_color(gtk.gdk.color_parse(pigm.secondaryColour))
                except ValueError:
                    print >> sys.stderr, "prim", pigm.primaryColour, "second", pigm.secondaryColour
                    raise
            elif pigm.type.codeName == "xsplashHack":
                for widg,prop in (("Gtk",pigm.preferredGtkThemes),("Icons",pigm.preferredIconThemes),("Cursors",pigm.preferredCursorThemes)):
                    text = "\n".join(prop)
                    self.wTree.get_widget("textviewPreferred" + widg).get_buffer().set_text(text)
                
    def save(self):
        pigmentType = self.listStorePigmentTypes.get_value(\
            self.listStorePigmentTypes.get_iter(self.comboboxPigmentType.get_active()), 0)
        
        pigm = managepigments.create_pigment(pigmentType)
        pigm.set_human_name(self.entryName.get_text())
        pigm.codeName = self.entryCodename.get_text().strip()
        buff = self.textviewDescription.get_buffer()
        tt = buff.get_text(buff.get_iter_at_offset(0), buff.get_iter_at_offset(buff.get_char_count()))
        pigm.set_description(tt)
        pigm.copyright = self.comboboxentryCopyright.get_child().get_text()
        pigm.author = self.entryAuthor.get_text()
        
        # files and preview
        previewFile = self.filechooserbuttonPreview.get_filename()
        if previewFile is None:
            if self.pigmentPreviewPx[pigmentType] is None and pigmentType != "skin":
                self.show_warning(_("Please provide a preview image"))
                return
            elif not self.pigmentPreviewPx[pigmentType] is None:
                previewFile = "/tmp/epidermis/creator/preview-" + pigmentType + ".png"
                subprocess.call(["mkdir", "-p", "/tmp/epidermis/creator/"])
                self.pigmentPreviewPx[pigmentType].save(previewFile, "png")
        # now we've got a filename for preview
        try:
            if not previewFile is None:
                pigm.set_preview_file(previewFile) #.split("/")[-1]
        except gobject.GError as ee:
            self.show_warning(_("Preview image is corrupt"))
            print >> sys.stderr, str(ee)
        themeFile = self.filechooserbuttonFile.get_filename()
        if pigmentType != "skin" and themeFile is None:
            self.show_warning(_("Please provide a theme file or directory"))
            return
        if pigmentType in ("wallpaper","gnomeSplash","grub"):
            fileOrDir = "filename"
        else:
            fileOrDir = "dirname"
        if not themeFile is None and pigmentType != "skin":
            pigm.add_attachment(themeFile)
        
        getw = self.wTree.get_widget
        def make_list(words):
            ll = words.split(",")
            for ii in xrange(len(ll)):
                ll[ii] = ll[ii].lower()
            return ll
        if pigmentType == "skin":
            pigm.primaryColours = make_list(getw("entryPrimaryColours").get_text())
            pigm.secondaryColours = make_list(getw("entrySecondaryColours").get_text())
            pigm.keyWords["en"] = make_list(getw("entryKeyWords").get_text())
            pigm.version = getw("entryVersion").get_text()
            pigm.webPage = getw("entryWebpage").get_text()
            for bit in const.PIGMENT_TYPES[1:]:
                codename = self.pigment_selector_get_text(getw("button" + bit.capitalize()))
                if codename != "" and not codename is None:
                    pigm.set_linked_pigment(managepigments.load_pigment( \
                        self.pigment_selector_get_text(getw("button" + bit.capitalize())), bit))
        elif pigmentType == "wallpaper":
            pigm.pictureOptions = self.listStorePictureOptions.get_value( \
                self.listStorePictureOptions.get_iter(self.comboboxPictureOptions.get_active()), 0)
            pigm.colourShadingType = self.listStoreColourShadingType.get_value( \
                self.listStoreColourShadingType.get_iter(self.comboboxColourShadingType.get_active()), 0)
            pigm.primaryColour = getw("colorbuttonPrimaryColour").get_color().to_string()
            pigm.secondaryColour = getw("colorbuttonSecondaryColour").get_color().to_string()
        elif pigmentType == "xsplashHack":
            for pref, pref2 in (("gtk","Gtk"),("icons","Icon"),("cursors","Cursor")):
                buff = self.wTree.get_widget("textviewPreferred" + pref.capitalize()).get_buffer()
                text = buff.get_text(buff.get_iter_at_offset(0), buff.get_iter_at_offset(buff.get_char_count()))
                setattr(pigm,"preferred" + pref2 + "Themes",[])
                for line in text.split("\n"):
                    if line.strip() != "":
                        getattr(pigm,"preferred" + pref2 + "Themes").append(line)
                
        
        
        # info is a dictionary to check if all the information has been entered
        info = {}
        pigmentType = self.listStorePigmentTypes.get_value(\
            self.listStorePigmentTypes.get_iter(self.comboboxPigmentType.get_active()), 0)
        info["name"] = self.entryName.get_text()
        info["pigment"] = self.entryCodename.get_text().strip()
        buff = self.textviewDescription.get_buffer()
        info["description"] = buff.get_text(buff.get_iter_at_offset(0), buff.get_iter_at_offset(buff.get_char_count()))
        info["copyright"] = self.comboboxentryCopyright.get_child().get_text()
        info["author"] = self.entryAuthor.get_text()
        
        # files
        info["preview"] = previewFile
        themeFile = self.filechooserbuttonFile.get_filename()
        if pigmentType != "skin" and themeFile is None:
            self.show_warning(_("Please provide a theme file or directory"))
            return
        if pigmentType in ("wallpaper","gnomeSplash","grub"):
            fileOrDir = "filename"
        else:
            fileOrDir = "dirname"
        info[fileOrDir] = themeFile
        
        getw = self.wTree.get_widget
        if pigmentType == "skin":
            info["primarycolours"] = getw("entryPrimaryColours").get_text()
            info["secondarycolours"] = getw("entrySecondaryColours").get_text()
            info["keywords"] = getw("entryKeyWords").get_text()
            info["version"] = getw("entryVersion").get_text()
            info["webpage"] = getw("entryWebpage").get_text()
            for bit in const.PIGMENT_TYPES[1:]:
                if self.pigment_selector_get_text(getw("button" + bit.capitalize())) is not None:
                    info[bit] = self.pigment_selector_get_text(getw("button" + bit.capitalize()))
        elif pigmentType == "wallpaper":
            info["pictureOptions"] = self.listStorePictureOptions.get_value( \
                self.listStorePictureOptions.get_iter(self.comboboxPictureOptions.get_active()), 0)
            info["colorShadingType"] = self.listStoreColourShadingType.get_value( \
                self.listStoreColourShadingType.get_iter(self.comboboxColourShadingType.get_active()), 0)
            info["primaryColor"] = getw("colorbuttonPrimaryColour").get_color().to_string()
            info["secondaryColor"] = getw("colorbuttonSecondaryColour").get_color().to_string()
        
        # in case information is not ready, display message
        fillInFor = ""
        humanInfoKeys = {
            'gnomeSplash': _("GNOME splash"), 'gtk':_("GTK"), 'xsplashHack':_("Xsplash"), 'dirname':_("Theme directory"),
            'description':_("Description"), 'copyright':_("Copyright"), 'author':_("Author"), 'webpage':_("Web page"),
            'wallpaper':_("Wallpaper"), 'cursors':("Mouse cursors"), 'grub':("Grub"), 'pigment':_("Codename"),
            'primarycolours':_("Primary Colors"), 'secondarycolours':_("Secondary Colors"), 'version':_("Version"),
            'keywords':_("Key Words"), 'icons':_("Icons"), 'preview':_("Preview"), 'gdm':_("GDM"), 'metacity':_("Metacity"),
            'name':_("Name"), 'pictureOptions':_("Picture Options"), 'colorShadingType':_("Color Shading Type"),
            'primaryColor':_("Primary Color"), 'secondaryColor':_("Secondary Color")}
        for bit in info.keys():
            doWarning = False
            if info[bit] is None:
                doWarning = True
            elif info[bit].strip() == "":
                doWarning = True
            if pigmentType == "skin" and bit.lower() == "dirname":
                doWarning = False
            if doWarning:
                fillInFor = fillInFor + humanInfoKeys[bit] + ", "
        if len(fillInFor) > 0:
            self.show_warning(_("Please fill in information for: %s") % fillInFor[:-2])
            return
        oneLinkedPigment = False
        for pt in const.PIGMENT_TYPES[1:]:
            if info.has_key(pt):
                oneLinkedPigment = True
                break
        if not oneLinkedPigment and pigmentType == "skin":
            self.show_warning(_("Skins must have at least one linked pigment"))
            return
        
        if len(pigm.attachments) > 0:
            if not pigm.is_attachment_correct():
                self.show_warning(_("Invalid theme file or directory: %s") % pigm.type.attachmentTypeHelp)
                return
        

        # Open GTK file dialog and save
        filter = gtk.FileFilter()
        filter.add_pattern("*.pigment")
        filter.set_name(_("pigments"))
        
        dd = gtk.FileChooserDialog(_("Save pigment"), None, gtk.FILE_CHOOSER_ACTION_SAVE, \
            buttons=(gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL, gtk.STOCK_SAVE, gtk.RESPONSE_ACCEPT))
        dd.add_filter(filter)
        dd.set_current_name(pigm.codeName + "_" + pigmentType + ".pigment")
        dd.set_current_folder(self.openDir)
        res = dd.run()
        ff = dd.get_filenames()
        self.openDir = dd.get_current_folder()
        dd.destroy()
        
        if len(ff[0]) < len(".pigment"):
            ff[0] = ff[0] + ".pigment"
        elif ff[0][-len(".pigment"):] != ".pigment":
            ff[0] = ff[0] + ".pigment"
        
        pigm.save(ff[0])
        
        
            
        
    
    def menuSave_callback(self, widget):
        self.save()
    
    def menuQuit_callback(self, widget):
        self.destroy_event(self.win, "menuQuit")
    
    def menuAbout_callback(self, widget):
        winAbout = self.wTree.get_widget("aboutdialog1")
        winAbout.set_version(const.VERSION)
        winAbout.set_translator_credits(_("translator-credits"))
        winAbout.run()
        winAbout.hide()
    
    
    def pigment_selector_set_text(self, widget, text):
        if text == "":
            text = "(None)"
        for child in widget.get_child().get_children():
            if isinstance(child, gtk.Label):
                child.set_text(text)
                break
        enoughPigments = False
        for ii in ("Wallpaper","Gtk","Metacity","Icons"):
            if not None is self.pigment_selector_get_text(self.wTree.get_widget("button" + ii)):
                enoughPigments = True
                break
        self.buttonAutogenerate.set_sensitive(enoughPigments)
    
    def pigment_selector_get_text(self, widget):
        """Get the text from a pigment selector widget"""
        for child in widget.get_child().get_children():
            if isinstance(child, gtk.Label):
                if child.get_text() == "(None)":
                    return None
                else:
                    return child.get_text()
    
    def pigment_selector_callback(self, widget, pigmentType):
    
        # prepare ListStore
        listStore = gtk.ListStore(gtk.gdk.Pixbuf, str, str)
        
        pt = pigmentType
        treeview = self.treeviewSelectPigment
        # delete all previous columns
        for col in treeview.get_columns():
            treeview.remove_column(col)
        # prepare columns
        colPic = gtk.TreeViewColumn(_("Preview"))
        colPig = gtk.TreeViewColumn(_("Pigment"))
        colNam = gtk.TreeViewColumn(_("Name"))
        crPic = gtk.CellRendererPixbuf()
        crPig = gtk.CellRendererText()
        crNam = gtk.CellRendererText()
        colPic.pack_start(crPic, True)
        colPic.add_attribute(crPic, "pixbuf", self.iSELECT_PICTURE)
        colPig.pack_start(crPig, True)
        colPig.add_attribute(crPig, "text", self.iSELECT_PIGMENT)
        colNam.pack_start(crNam, True)
        colNam.add_attribute(crNam, "text", self.iSELECT_NAME)
        treeview.set_model(listStore)
        treeview.append_column(colPic)
        treeview.append_column(colNam)
        treeview.append_column(colPig)
        
        pigms, excps = managepigments.find_pigments(pt)
        addedSomething = False
        for pigm in pigms:
            listStore.append( \
                [pigm.preview, pigm.get_human_name(), pigm.codeName] )
            addedSomething = True
        
        if addedSomething:
            self.butSelectOpen.set_sensitive(True)
            treeview.get_selection().select_path(0)
            res = self.winSelectPigment.run()
            self.winSelectPigment.hide()
            if res == 1:
                modTreeview, iterr = treeview.get_selection().get_selected()
                selected = listStore.get_value(iterr, self.iSELECT_PIGMENT)
                self.pigment_selector_set_text(widget, selected)
        else:
            self.butSelectOpen.set_sensitive(False)
            listStore.append([self.win.render_icon(globals()["gtk"].STOCK_DIALOG_ERROR, globals()["gtk"].ICON_SIZE_BUTTON), \
                _("No pigments found, please install some"), ""])
            self.winSelectPigment.run()
            self.winSelectPigment.hide()
    
    def preferred_callback(self,widget,data=None):
        # prepare ListStore
        listStore = gtk.ListStore(str)
        
        pt = data.lower()
        treeview = self.treeviewSelectTheme
        # delete all previous columns
        for col in treeview.get_columns():
            treeview.remove_column(col)
        # prepare column
        colTheme = gtk.TreeViewColumn(_("Theme"))
        cr = gtk.CellRendererText()
        colTheme.pack_start(cr, True)
        colTheme.add_attribute(cr, "text", 0)
        treeview.set_model(listStore)
        treeview.append_column(colTheme)
        
        if pt == "gtk":
            for ii in os.listdir("/usr/share/themes"):
                if os.path.isdir(os.path.join("/usr/share/themes",ii,"gtk-2.0")):
                    listStore.append([ii])
        elif pt == "icons":
            for ii in os.listdir("/usr/share/icons"):
                if not os.path.isdir(os.path.join("/usr/share/icons",ii)):
                    continue
                dirs = os.listdir(os.path.join("/usr/share/icons",ii))
                if "cursors" in dirs:
                    dirs.remove("cursors")
                if len(dirs) >= 2:
                    listStore.append([ii])
        elif pt == "cursors":
            for ii in os.listdir("/usr/share/icons"):
                if os.path.isdir(os.path.join("/usr/share/icons",ii,"cursors")):
                    listStore.append([ii])
        
        if len(listStore) >= 1:
            self.butSelectThemeOpen.set_sensitive(True)
            treeview.get_selection().select_path(0)
            res = self.winSelectTheme.run()
            self.winSelectTheme.hide()
            if res == 1:
                modTreeview, iterr = treeview.get_selection().get_selected()
                selected = listStore.get_value(iterr, 0)
                buff = self.wTree.get_widget("textviewPreferred" + pt.capitalize()).get_buffer()
                buff.set_text(buff.get_text(buff.get_iter_at_offset(0),buff.get_iter_at_offset(buff.get_char_count())) + selected)
        else:
            self.butSelectThemeOpen.set_sensitive(False)
            listStore.append([_("No themes found, please install some")])
            self.winSelectTheme.run()
            self.winSelectTheme.hide()        
    
    def entryName_callback(self, widget, data=None):
        if self.entryCodename.get_text() == "":
            self.codenameAutomatic = True
        if self.codenameAutomatic:
            self.codenameManual = True
            self.entryCodename.set_text(self.entryName.get_text().replace(" ", "-").lower())
            self.codenameManual = False
    
    def entryCodename_callback(self, widget, data=None):
        if not self.codenameManual:
            self.entryCodename.set_text(self.entryCodename.get_text().replace(" ", "-").lower())
            self.codenameAutomatic = False
    
    def comboboxPigmentType_callback(self, widget, data=None):
        pigmentType = self.listStorePigmentTypes.get_value(\
            self.listStorePigmentTypes.get_iter(self.comboboxPigmentType.get_active()), 0)
        
        # show controls from winOther
        widgets = {"skin":"tableSkin", "wallpaper":"tableWallpaper","xsplashHack":"tableXsplashhack"}
        # fill in non specified widgets:
        for pt in const.PIGMENT_TYPES:
            if not widgets.has_key(pt):
                widgets[pt] = "lbl" + pt.capitalize()
    
        notebook = self.wTree.get_widget("notebookOther")
        if not self.currentFrameSpecificChild is None:
            self.frameSpecific.remove(self.currentFrameSpecificChild)
            notebook.add(self.currentFrameSpecificChild)
        if self.wTree.get_widget(widgets[pigmentType]) is None:
            print >> sys.stderr, "couldn't get widget:%s for pt:%s" % (widgets[pigmentType], pigmentType)
        self.currentFrameSpecificChild = self.wTree.get_widget(widgets[pigmentType])
        notebook.remove(self.currentFrameSpecificChild)
        self.frameSpecific.add(self.currentFrameSpecificChild)
        
        # set theme file to file or dir
        if pigmentType in ("wallpaper","gnomeSplash","grub"):
            # file
            self.labelFile.set_label("Theme file:")
            try:
                if not os.path.isfile(self.filechooserbuttonFile.get_filename()):
                    self.filechooserbuttonFile.unselect_all()
            except TypeError: pass
            self.filechooserbuttonFile.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
            self.filechooserbuttonFile.set_sensitive(True)
        elif pigmentType == "skin":
            self.labelFile.set_label("<span strikethrough=\"true\">Theme file:</span>")
            self.filechooserbuttonFile.unselect_all()
            self.filechooserbuttonFile.set_sensitive(False)
        else:
            # directory
            self.labelFile.set_label("Theme directory:")
            self.filechooserbuttonFile.set_sensitive(True)
            try:
                if not os.path.isdir(self.filechooserbuttonFile.get_filename()):
                    self.filechooserbuttonFile.unselect_all()
            except TypeError: pass
            self.filechooserbuttonFile.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
        
        # enabling or disabling buttonAutogenerate
        type = managepigments.get_pigment_type(pigmentType)
        self.buttonAutogenerate.set_sensitive(type.canGeneratePreview)
        if pigmentType == "skin":
            enoughLinkedPigments = False
            for ii in ("Wallpaper","Metacity","Gtk","Icons"):
                if not None is self.pigment_selector_get_text(self.wTree.get_widget("button" + ii)):
                    enoughLinkedPigments = True
                    break
            if not enoughLinkedPigments:
                self.buttonAutogenerate.set_sensitive(False)
        
        # reset preview file
        self.imagePreview.set_from_pixbuf(self.pigmentPreviewPx[pigmentType])
        self.filechooserbuttonPreview.unselect_all()
        
        # handling sizeGroup
        self.sizeGroup0 = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        self.sizeGroup1 = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
        if isinstance(self.currentFrameSpecificChild, gtk.Table):
            # add upper right widgets to sizeGroup1
            for nn in ("comboboxPigmentType", "entryName", "entryCodename", \
            "comboboxentryCopyright", "entryAuthor", "filechooserbuttonFile", \
            "scrolledwindowDescription", "imagePreview", "buttonAutogenerate", \
            "filechooserbuttonPreview"):
                self.sizeGroup1.add_widget(self.wTree.get_widget(nn))
            # add upper left widgets to sizeGroup0
            for nn in ("Type","Name", "Codename", "Copyright", "Author",  "File", "Description", \
            "Image", "Preview"):
                self.sizeGroup0.add_widget(self.wTree.get_widget("label" + nn))
            # add lower labels to sizeGroup0 , and other lower widgets to sizeGroup1
            def do_this(widgett):
                if isinstance(widgett, gtk.Label):
                    self.sizeGroup0.add_widget(widgett)
                else:
                    self.sizeGroup1.add_widget(widgett)
            self.currentFrameSpecificChild.foreach(do_this)
        self.wTree.get_widget("vbox2222").check_resize()
    
    def filechooserbuttonPreview_callback(self, widget):
        """update imagePreview with the image selected"""
        pf = self.filechooserbuttonPreview.get_filename()
        pigmentType = self.listStorePigmentTypes.get_value(\
            self.listStorePigmentTypes.get_iter(self.comboboxPigmentType.get_active()), 0)
        if pf is None:
            self.imagePreview.set_from_stock(gtk.STOCK_MISSING_IMAGE,  gtk.ICON_SIZE_SMALL_TOOLBAR)
            self.pigmentPreviewPx[pigmentType] = None
        else:
            px = gtk.gdk.pixbuf_new_from_file_at_size(pf, 200, 150)
            self.pigmentPreviewPx[pigmentType] = gtk.gdk.pixbuf_new_from_file(pf)
            self.imagePreview.set_from_pixbuf(px)
    
    def buttonAutogenerate_callback(self, widget):
        pigmentType = self.listStorePigmentTypes.get_value(\
            self.listStorePigmentTypes.get_iter(self.comboboxPigmentType.get_active()), 0)
        if pigmentType != "skin" and self.filechooserbuttonFile.get_filename() is None:
            self.show_warning(_("Please provide a theme file or directory"))
            self.imagePreview.set_from_stock(gtk.STOCK_MISSING_IMAGE,  gtk.ICON_SIZE_SMALL_TOOLBAR)
            self.pigmentPreviewPx[pigmentType] = None
            self.filechooserbuttonPreview.unselect_all()
            return
        self.buttonAutogenerate.set_sensitive(False)
        pigm = managepigments.create_pigment(pigmentType)
        if pigmentType == "skin":
            for ii in ("wallpaper","gtk","icons","metacity"):
                linkedPigmName = self.pigment_selector_get_text(self.wTree.get_widget("button" + ii.capitalize()))
                if not linkedPigmName is None:
                    linkedPigm = managepigments.load_pigment(linkedPigmName, ii)
                    pigm.set_linked_pigment(linkedPigm)
        else:
            pigm.add_attachment(self.filechooserbuttonFile.get_filename())
            if not pigm.is_attachment_correct():
                self.show_warning(_("Invalid theme file or directory: %s") % pigm.type.attachmentTypeHelp)
                self.imagePreview.set_from_stock(gtk.STOCK_MISSING_IMAGE,  gtk.ICON_SIZE_SMALL_TOOLBAR)
                self.pigmentPreviewPx[pigmentType] = None
                self.filechooserbuttonPreview.unselect_all()
                self.buttonAutogenerate.set_sensitive(True)
                return
        self.win.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.WATCH))
        px = pigm.generate_preview()
        self.win.window.set_cursor(None)
        self.imagePreview.set_from_pixbuf(px)
        self.pigmentPreviewPx[pigmentType] = px
        self.filechooserbuttonPreview.unselect_all()
        self.buttonAutogenerate.set_sensitive(True)
    
    def get_display_name_of_pigment_type(self, pigmentType):
        return managepigments.get_pigment_type(pigmentType)().get_human_name()
    
    def show_warning(self, text, secondaryText=None):
        dialog = gtk.MessageDialog(None, gtk.DIALOG_DESTROY_WITH_PARENT | gtk.DIALOG_MODAL, \
            gtk.MESSAGE_WARNING, gtk.BUTTONS_OK, text)
        dialog.set_transient_for(self.win)
        dialog.set_destroy_with_parent(True)
        if not secondaryText is None:
            dialog.format_secondary_text(secondaryText)
        dialog.run()
        gobject.idle_add(lambda:dialog.hide())

def determine_path ():
    """Borrowed from wxglade.py"""
    try:
        root = __file__
        if os.path.islink (root):
            root = os.path.realpath (root)
        return os.path.dirname (os.path.abspath (root))
    except:
        print >> sys.stderr, "I'm sorry, but something is wrong."
        print >> sys.stderr, "There is no __file__ variable. Please contact the author."
        sys.exit(1)

def main():
    # gtk stuff
    import webbrowser
    gtk.about_dialog_set_url_hook(lambda dialog, link, data=None: webbrowser.open(link))


    # i18n stuff
    locale.setlocale(locale.LC_ALL, '')
    if const.DEBUG: print "locale.getlocale()", locale.getlocale()
    if os.path.exists(os.path.join("/".join(determine_path().split("/")[:-1]), const.LOCALE_DIR)):
        print "Using custom locale dir"
        locale_dir = os.path.join("/".join(determine_path().split("/")[:-1]), const.LOCALE_DIR)
    else:
        locale_dir = "/usr/share/locale"
    for module in (gtk.glade, gettext):
        module.bindtextdomain(const.APP_NAME, locale_dir)
        module.textdomain(const.APP_NAME)
    
    argv = sys.argv[1:]
    if '--debug' in argv:
        import debuginfo
        print debuginfo.debuginfo()
        const.DEBUG = True
        argv.pop(argv.index("--debug"))
    if '--creator' in argv:
        argv.pop(argv.index('--creator'))
    
    app = CreatorApp()
    threading.currentThread().setName(const.GTK_LOOP_THREAD_NAME)
    if len(argv) > 0:
        gobject.idle_add(lambda:app.open(argv))
    gtk.main()

if __name__ == "__main__":
    main()

